# -*- coding: utf-8 -*-
from functools import wraps
import json
from vra_sdk.vra_exceptions import VraSdkRequestException, VraSdkEntitlementException, VraSdkDecoratorException
import requests


def check_entitlement(func):
    """Check if the requested action/catalog item is in the catalog
    
    Args:
        func (function): request function to checj
    
    Raises:
        VraSdkEntitlementException: Requested action/catalog item is not in the catalog
    
    Returns:
        func: decorated function
    """


    @wraps(func)
    def wrapper(*args, **kwargs):
        # args[0] is the first argument of the decorated function, so it's the self of the decorated function
        if args[1] in args[0].catalog:
            return func(*args, **kwargs)
        else:
            raise VraSdkEntitlementException(
                'You do not have permissions to perform this request')
    return wrapper


def update_catalog_resource_operation(func):
    """Update the catalog to add the resouce action
    
    Args:
        func (function): decorated function
    
    Raises:
        VraSdkRequestException: Raise if any vRa infrastructure connection error
        VraSdkDecoratorException:  Unmanaged error
    
    Returns:
        function: decorated function
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        # args[0] is the first argument of the decorated function, so it's the self of the decorated function
        # args[1] is the vRa name of the day2 operation
        # args[2] is the id of the resource
        if args[1] not in args[0].catalog:
            try:
                req = args[0].config.session.get(
                    f'https://{args[0].config.vcac_server}/catalog-service/api/consumer/resources/{args[2]}', verify=args[0].config.verify, timeout=args[0].config.timeout)
                req.raise_for_status()
                response = json.loads(req.text)

                for elt in response['operations']:
                    args[0].catalog[elt["name"]] = elt["id"]
            except requests.exceptions.RequestException as e:
                raise VraSdkException(
                    f'Error retrieving resource operation: {e}')
            except Exception as e:
                raise VraSdkDecoratorException(e)
        return func(*args, **kwargs)
    return wrapper


def singleton(cls):
    """Decorator to make singletonclass 
    This decorator doesn't work for classes that implements the __new__ method.
    To re-enable the init of the class, set the _sealed attribute to False
    """

    class SingleClass(cls):
        """ The real singleton. """
        _instance = None
        __module__ = cls.__module__
        __doc__ = cls.__doc__

        def __new__(cls, *args, **kwargs):
            if SingleClass._instance is None:
                SingleClass._instance = super().__new__(cls)
                SingleClass._instance._sealed = False

            return SingleClass._instance

        def __init__(self, *args, **kwargs):
            if not getattr(self, '_sealed', False):
                super().__init__(*args, **kwargs)
                self._sealed = True

    SingleClass.__name__ = cls.__name__
    return SingleClass



# -*- coding: utf-8 -*-
import json
import inspect


class VraBaseObject:
    """Base class for business models class. Use to implement basic method.
    """

    def to_dict(self, raw_data=False):
        """Return serialized object
            raw_data (bool, optional): Defaults to False. If true, also return the raw_data attribute of the object
        
        Returns:
            dict: dict of the object attribute
        """

        result = {}
        for k in inspect.signature(self.__class__).parameters:
            result[k] = getattr(self, k)

        if not raw_data:
            result.pop('raw_data')
        return result

    def to_json(self, raw_data=False):
        """Return json representation of the object
            raw_data (bool, optional): Defaults to False. If true, also return the raw_data attribute of the object
        
        Returns:
            string: json representation of the object
        """

        return json.dumps(self.to_dict(raw_data))



# -*- coding: utf-8 -*-

import json
import requests
import os
from vra_sdk.vra_decorator import singleton
from vra_sdk.vra_utils import resolve_path
from vra_sdk.vra_exceptions import VraSdkConfigException, VraSdkMainConfigException


@singleton
class VraConfig():
    """Handle configuration loading

    Warning: This class is a singleton created through the vra_sdk.vra_decorator singleton method
     
    Attributes:
        config_file (dict): Serialization of the configuration file
        verify (boolean): Requests verify option behavior
        session (requests.sessions): Requests session object
        vcac_server (string): vRa server
    """

    def __init__(self, config_path=None):
        """Init VraConfig
            config_path (string, optional): Defaults to None. Path to the config file. Relative to the python execution path
        
        Raises:
            VraSdkConfigException: Config file not found
            VraSdkMainConfigException: Unmanaged error
        """

        self.config_file={}
        if config_path:
            try:
                final_config_path = resolve_path(config_path)
                with open(final_config_path, 'r') as f:
                    self.config_file = json.load(f)
            except FileNotFoundError as e:
                raise VraSdkConfigException(f'Config file {final_config_path} not found {e}')
            except Exception as e:
                raise VraSdkMainConfigException(e)

        self.verify = self.config_file.get('verify', True)
        self.timeout = self.config_file.get('timeout', 30)
        self.session = requests.Session()
        self.session.trust_env = False
        self.vcac_server = None



# -*- coding: utf-8 -*-
from vra_sdk.vra_utils import get_module_class, clean_kwargs_key
from vra_sdk.vra_config import VraConfig
import inspect
from vra_sdk.vra_exceptions import VraSdkFactoryException, VraSdkConfigException


class VraFactory(object):
    """Factory to create specific object class
    """

    
    

    @staticmethod
    def factory(object_type, customization_func=None, **kwargs):
        """Factory to create specific type object
        
        Args:
            object_type (string): Object type to create as defined in the business_models section of the configuration file
            customization_func (func, optional): Defaults to None. For 'payload' type only. Post customization payload processing function
        
        Returns:
            object: object of the 'object_type' type specified in args
        """

        config = VraConfig().config_file
        enforce_id = None

        if object_type == 'payload':
            if all(k in kwargs for k in ("payload_version", "payload_type")):
                str_version = str(kwargs['payload_version'])
                payload_type = kwargs['payload_type']
                object_path = f'vra_sdk.models.vra_payload_{str_version}.{payload_type}'
            else:
                raise VraSdkFactoryException(
                    "Error creating payload object. Missing required parameters")
        elif object_type in config['business_models']:
            object_path = config.get(
                'business_models').get(object_type).get('path')
            if not object_path:
                raise VraSdkFactoryException(
                    f"Error retrieving module_class for {object_type} object type.")
            enforce_id = config['business_models'][object_type].get('enforce_id')
        else:
            raise VraSdkConfigException(
                'Error building vraObject, unknown type')

        _, object_class = get_module_class(object_path)
        id_cards = inspect.signature(object_class).parameters
        cleaned_kwargs = clean_kwargs_key(**kwargs)

        if enforce_id:
            for kwarg in cleaned_kwargs:
                if kwarg not in id_cards:
                    raise VraSdkConfigException(
                        f"Error creating vraObject {object_type}, {kwarg} not authorized by the id card of {object_path}")

        if object_type == 'payload':
            to_return = object_class(customization_func, **cleaned_kwargs)
        else:
            to_return = object_class(**cleaned_kwargs)
        return to_return



from setuptools import setup
from setuptools.command.install import install


setup(name='vra_sdk',
      version="1.0.1",
      description='vra_sdk',
      url='https://github.com/richarddevers/vra_sdk',
      author='Richard DEVERS',
      author_email='ritchiedev25@gmail.com',
      license='MIT',
      include_package_data=True,
      packages=['vra_sdk', 'vra_sdk.models'],
      install_requires=[
          "pbr", 'requests', 'dateutils', 'urllib3'
      ],
      zip_safe=False
      )
